import re as _re

_valid_word = _re.compile(r"\b[a-zA-Z]+'?s?\b")


def get_common_size(x_list: list, y_list: list, x_sorted=False, y_sorted=False) -> int:
  if not x_sorted:
    x_list = sorted(x_list)
  if not y_sorted:
    y_list = sorted(y_list)
  cur, n, match = 0, len(y_list), 0
  for x in x_list:
    while cur < n and y_list[cur] < x:
      cur += 1
    if cur < n and y_list[cur] == x:
      cur += 1
      match += 1
  return match


def get_dice_coefficient(x_list: list, y_list: list, x_sorted=False, y_sorted=False) -> float:
  match = get_common_size(x_list, y_list, x_sorted, y_sorted)
  return 2 * match / (len(x_list) + len(y_list))


def is_subset_of(x_list: list, y_list: list, x_sorted=False, y_sorted=False) -> bool:
  if len(x_list) > len(y_list):
    return False
  if not x_sorted:
    x_list = sorted(x_list)
  if not y_sorted:
    y_list = sorted(y_list)
  cur, n = 0, len(y_list)
  for x in x_list:
    while cur < n and y_list[cur] < x:
      cur += 1
    if cur < n and y_list[cur] == x:
      cur += 1
    else:
      return False
  return True


def _normalize_word(word, equivalent_words):
  word = word.lower()
  for pair in equivalent_words:
    if word == pair['from']:
      return pair['to']
  return word

def split_and_normalize(text, equivalent_words: list=[], sort: bool=False):
  words = _valid_word.findall(text)
  words = [_normalize_word(word, equivalent_words) for word in words]
  if sort:
    words.sort()
  return words



import re
import difflib
from pprint import pprint


class DifferenceGenerator:
    """
    This Class generates the potential differences which are more than
    a length of 10 between the input text and the original input license
    text/template.
    """
    def __init__(self, constant_license, license_to_match):
        self.constant_license = constant_license
        self.license_to_match = license_to_match
        copy_license = license_to_match
        self.copy_license = copy_license
        self.copy_license = self.generate_difference(copy_license)

    def generate_difference(self, copy_license):

        matches = difflib.SequenceMatcher(None, self.constant_license, self.license_to_match).get_matching_blocks()
        print("Following text produces a mismatch with the original template/text")

        for match in matches:
            x = self.license_to_match[match.b:match.b + match.size]
            if (match.size > 10):
                copy_license = copy_license.replace(x, '')

        copy_license = copy_license.replace('`', ' ')
        return copy_license

    def pretty_print_differences(self):
        return self.copy_license



import os
import re
from .template import Node, SequentialNode, TextNode, OptionalNode, VarNode
import terregex


class Normalizer:
  def __call__(self, node: Node, *, pred: Node = None, succ: Node = None, parent: Node = None) -> Node:
    if isinstance(node, SequentialNode):
      return self.normalize_sequential(node, pred=pred, succ=succ, parent=parent)
    if isinstance(node, TextNode):
      return self.normalize_text(node, pred=pred, succ=succ, parent=parent)
    if isinstance(node, OptionalNode):
      return self.normalize_optional(node, pred=pred, succ=succ, parent=parent)
    if isinstance(node, VarNode):
      return self.normalize_var(node, pred=pred, succ=succ, parent=parent)
    raise NotImplementedError("Unsupported node type")

  def normalize_sequential(self,
                           node: SequentialNode,
                           *,
                           pred: Node = None,
                           succ: Node = None,
                           parent: Node = None) -> Node:
    preds = [pred] + node.nodes[:-1]
    succs = node.nodes[1:] + [succ]
    node.nodes = [self(n, pred=p, succ=s, parent=node) for p, n, s in zip(preds, node.nodes, succs)]
    return node.simplify()

  def normalize_text(self, node: TextNode, *, pred: Node = None, succ: Node = None, parent: Node = None) -> Node:
    return node

  def normalize_optional(self,
                         node: OptionalNode,
                         *,
                         pred: Node = None,
                         succ: Node = None,
                         parent: Node = None) -> Node:
    node.content = self(node.content, pred=pred, succ=succ, parent=parent)
    return node

  def normalize_var(self, node: VarNode, *, pred: Node = None, succ: Node = None, parent: Node = None) -> Node:
    node.original = self(node.original, pred=pred, succ=succ, parent=parent)
    return node


class SequentialNormalizer(Normalizer):
  def __init__(self, *normalizers: list[Normalizer]):
    self.normalizers = normalizers

  def __call__(self, node: Node, *, pred: Node = None, succ: Node = None, parent: Node = None) -> Node:
    for normalizer in self.normalizers:
      node = normalizer(node, pred=pred, succ=succ, parent=parent)
    return node


class LowercaseNormalizer(Normalizer):
  """
  Converts the string into lowercase.
  """

  def __init__(self):
    self.normalize_regex = terregex.Transformer()

    @self.normalize_regex.add_rule()
    def rule1(literal: terregex.Literal):
      literal.string = literal.string.lower()

  def normalize_text(self, node: TextNode, **_) -> Node:
    node.text = node.text.lower()
    return node

  def normalize_var(self, node: VarNode, **_):
    node.pattern = self.normalize_regex(node.pattern)
    return node


class EquivalentWordsNormalizer(Normalizer):
  """
  Checks each word against the equivalent words
  list to avoid mismatch of equivalent words.
  """

  def __init__(self, equivalentwords) -> None:
    self.equivalentwords = equivalentwords

  def normalize_text(self, node: TextNode, **_) -> Node:
    for rule in self.equivalentwords:
      node.text = node.text.replace(rule['from'], rule['to'])
    return node


class CopyrightSymbolNormalizer(Normalizer):
  """
  Removes the copyright symbol and the possibilities 
  of mismatch due to it.
  """

  def normalize_text(self, node: TextNode, **_) -> Node:
    node.text = node.text.replace("copyright", '')
    node.text = node.text.replace('(c)', '')
    return node


class BulletsNumberingNormalizer(Normalizer):
  """
  The most important and error prone function is the handling
  of the bullets and numbering cases. The case has just 1 error
  prone possibility and requirement that the numbering should
  carry a space after it or else it gets matched with the
  version. (1. ) will match while (1.) will not.
  """

  def normalize_text(self, node: TextNode, **_) -> Node:
    regex_to_substitute = [
        r'([0-9]+\.){2,}',
        r'[0-9]+\.[\D]',
        r'^[a-z]\.',
        r'[a-z]\)',
        r'[0-9]\)',
        r'^[A-Z]\.',
        r'^[mdclxvi]+\.',
    ]
    for x in regex_to_substitute:
      node.text = re.sub(x, '', node.text)
    return node


class PunctuationNormalizer(Normalizer):
  """
  Replaces the common punctuations with ` to avoid errors.
  """
  _pattern_period = re.compile(r'\.(?=[a-z])')
  _pattern_comma = re.compile(r'\,(?=[a-z])')
  _pattern_hyphen = re.compile(r'\-(?=[a-z])')

  def normalize_text(self, node: TextNode, **_) -> Node:
    punctuations = ['/', '\'', '\"', '`']
    node.text = node.text.replace('_', '-')
    node.text = node.text.replace('--', '-')

    for x in punctuations:
      node.text = node.text.replace(x, '`')

    node.text = self._pattern_period.sub('.`', node.text)
    node.text = self._pattern_comma.sub(',`', node.text)
    node.text = self._pattern_hyphen.sub('-`', node.text)
    return node


class LicenseTitleNormalizer(Normalizer):
  def normalize_text(self, node: TextNode, **_) -> Node:
    node.text = node.text.replace('end of terms and conditions', '')
    return node


class WhiteSpaceNormalizer(Normalizer):
  """
  All the Whitespace and the tabs are removed. At the end of
  this function the input file just becomes a single long
  string which is easier to match.
  """
  _optional_space = OptionalNode(TextNode('`'))

  def __init__(self):
    self.normalize_regex = terregex.Transformer()

    @self.normalize_regex.add_rule()
    def rule1(literal: terregex.Literal):
      literal.string = literal.string.replace(' ', '`')

  def normalize_text(self, node: TextNode, *, pred: Node = None, succ: Node = None, **_) -> Node:
    if isinstance(pred, OptionalNode) or isinstance(succ, VarNode):
      node.trim(r=False)
    if isinstance(succ, OptionalNode) or isinstance(succ, VarNode):
      node.trim(l=False)
    node.text = re.sub(r'\s+', '`', node.text)
    node.text = re.sub(r'\`+', '`', node.text)
    return node

  def normalize_optional(self, node: OptionalNode, **_) -> Node:
    node.content = node.content.trim()
    node.content = self(node.content)
    return SequentialNode(self._optional_space, node, self._optional_space)

  def normalize_var(self, node: VarNode, **_):
    node.pattern = self.normalize_regex(node.pattern)
    return SequentialNode(self._optional_space, node, self._optional_space)


class Trimmer(Normalizer):
  """
  All leading/trailing spaces are removed.
  """

  def __init__(self, char: str = None, l: bool = True, r: bool = True):
    self.char = char
    self.l = l
    self.r = r

  def __call__(self, node: Node, **_) -> Node:
    return node.trim(char=self.char, l=self.l, r=self.r)


class TemplateSimplifier(Normalizer):
  """
  Structure of a template is simplified as much as possible.
  """

  def __call__(self, node: Node, **_) -> Node:
    return node.simplify()


class TextNormalizer(SequentialNormalizer):
  def __init__(self, equivalentwords=[]) -> None:
    super().__init__(
        LowercaseNormalizer(),
        EquivalentWordsNormalizer(equivalentwords),
        CopyrightSymbolNormalizer(),
        BulletsNumberingNormalizer(),
        PunctuationNormalizer(),
        LicenseTitleNormalizer(),
        WhiteSpaceNormalizer(),
        Trimmer(char='`'),
    )


class TemplateNormalizer(SequentialNormalizer):
  def __init__(self, equivalentwords=[]) -> None:
    super().__init__(
        LowercaseNormalizer(),
        EquivalentWordsNormalizer(equivalentwords),
        PunctuationNormalizer(),
        WhiteSpaceNormalizer(),
        Trimmer(char='`'),
        TemplateSimplifier(),
    )



from yalm._resources import ResourceLoader as _ResourceLoader, \
  KeyedDocument as _KeyedDocument

from yalm.template.template import Node as _Node
import re as _re
from typing import Union as _Union
from yalm.template import normalizer as _normalizer, template as _template
from yalm.words import split_and_normalize as _split_and_normalize, \
    is_subset_of as _is_subset_of
from collections import OrderedDict as _OrderedDict

class SpdxLicense:
  def __init__(self, identifier: str, name: str, priority: float, resources: _ResourceLoader) -> None:
    self.id = identifier
    self.name = name
    self.priority = priority
    self._resources = resources

  @property
  def template(self) -> _Node:
    return self._resources.get_template(self.id)

  @property
  def regex(self) -> _re.Pattern:
    return self._resources.get_regex(self.id)

  @property
  def words(self) -> list[str]:
    return self._resources.get_words(self.id)

  @property
  def positive_samples(self) -> list[_KeyedDocument]:
    return self._resources.get_positive_samples(self.id)

  def positive_sample(self, key) -> _KeyedDocument:
    return self._resources.get_positive_sample(self.id, key)

  def _test_words(self, words: list[str], words_sorted: bool=False) -> _Union[float, bool]:
    return _is_subset_of(self.words, words, x_sorted=True, y_sorted=words_sorted)

  def _test_regex(self, text: str) -> bool:
    return self.regex.fullmatch(text) is not None

class LicenseMatch:
  def __init__(self, text, template):
    self.text = text
    self.template = template

_resources = _ResourceLoader()
_normalizer = _normalizer.TemplateNormalizer(_resources.equivalent_words)

# resource loading
spdx_licenses: _OrderedDict[str, SpdxLicense] = {
  item['id']: SpdxLicense(item['id'], item['name'], item['priority'], _resources) for item in _resources.index
}


def _normalize_text(text: str) -> str:
  return _normalizer(_template.TextNode(text)).text

def detect_license(text: str) -> LicenseMatch:
  words = _split_and_normalize(text, _resources.equivalent_words, sort=True)
  normalized_text = _normalize_text(text)
  for license in spdx_licenses.values():
    if license._test_words(words, words_sorted=True) and license._test_regex(normalized_text):
      return LicenseMatch(text, license)
  return None



from yalm import licenses

mit, apache, *_ = list(licenses.spdx_licenses.values())

template = apache
key = '9409774ee89c484fe0f4b65d12bda1d2923d4687686858679172d473cd83038c.txt'

doc = licenses._normalize_text(template.positive_sample(key).document)
pattern = apache.regex.pattern



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['yalm',
 'yalm.resources',
 'yalm.resources.regex',
 'yalm.resources.template',
 'yalm.resources.tests',
 'yalm.resources.tests.classfier',
 'yalm.resources.tests.classfier.positive',
 'yalm.resources.tests.classfier.positive.0BSD',
 'yalm.resources.tests.classfier.positive.AAL',
 'yalm.resources.tests.classfier.positive.ADSL',
 'yalm.resources.tests.classfier.positive.AFL_1_1',
 'yalm.resources.tests.classfier.positive.AFL_1_2',
 'yalm.resources.tests.classfier.positive.AFL_2_0',
 'yalm.resources.tests.classfier.positive.AFL_2_1',
 'yalm.resources.tests.classfier.positive.AFL_3_0',
 'yalm.resources.tests.classfier.positive.AGPL_1_0',
 'yalm.resources.tests.classfier.positive.AGPL_1_0_only',
 'yalm.resources.tests.classfier.positive.AGPL_1_0_or_later',
 'yalm.resources.tests.classfier.positive.AGPL_3_0',
 'yalm.resources.tests.classfier.positive.AGPL_3_0_only',
 'yalm.resources.tests.classfier.positive.AGPL_3_0_or_later',
 'yalm.resources.tests.classfier.positive.AMDPLPA',
 'yalm.resources.tests.classfier.positive.AML',
 'yalm.resources.tests.classfier.positive.AMPAS',
 'yalm.resources.tests.classfier.positive.ANTLR_PD',
 'yalm.resources.tests.classfier.positive.ANTLR_PD_fallback',
 'yalm.resources.tests.classfier.positive.APAFML',
 'yalm.resources.tests.classfier.positive.APL_1_0',
 'yalm.resources.tests.classfier.positive.APSL_1_0',
 'yalm.resources.tests.classfier.positive.APSL_1_1',
 'yalm.resources.tests.classfier.positive.APSL_1_2',
 'yalm.resources.tests.classfier.positive.APSL_2_0',
 'yalm.resources.tests.classfier.positive.Abstyles',
 'yalm.resources.tests.classfier.positive.Adobe_2006',
 'yalm.resources.tests.classfier.positive.Adobe_Glyph',
 'yalm.resources.tests.classfier.positive.Afmparse',
 'yalm.resources.tests.classfier.positive.Aladdin',
 'yalm.resources.tests.classfier.positive.Apache_1_0',
 'yalm.resources.tests.classfier.positive.Apache_1_1',
 'yalm.resources.tests.classfier.positive.Apache_2_0',
 'yalm.resources.tests.classfier.positive.Artistic_1_0',
 'yalm.resources.tests.classfier.positive.Artistic_1_0_Perl',
 'yalm.resources.tests.classfier.positive.Artistic_1_0_cl8',
 'yalm.resources.tests.classfier.positive.Artistic_2_0',
 'yalm.resources.tests.classfier.positive.BSD_1_Clause',
 'yalm.resources.tests.classfier.positive.BSD_2_Clause',
 'yalm.resources.tests.classfier.positive.BSD_2_Clause_FreeBSD',
 'yalm.resources.tests.classfier.positive.BSD_2_Clause_NetBSD',
 'yalm.resources.tests.classfier.positive.BSD_2_Clause_Patent',
 'yalm.resources.tests.classfier.positive.BSD_2_Clause_Views',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_Attribution',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_Clear',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_LBNL',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_Modification',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_No_Military_License',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_No_Nuclear_License',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_No_Nuclear_License_2014',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_No_Nuclear_Warranty',
 'yalm.resources.tests.classfier.positive.BSD_3_Clause_Open_MPI',
 'yalm.resources.tests.classfier.positive.BSD_4_Clause',
 'yalm.resources.tests.classfier.positive.BSD_4_Clause_Shortened',
 'yalm.resources.tests.classfier.positive.BSD_4_Clause_UC',
 'yalm.resources.tests.classfier.positive.BSD_Protection',
 'yalm.resources.tests.classfier.positive.BSD_Source_Code',
 'yalm.resources.tests.classfier.positive.BSL_1_0',
 'yalm.resources.tests.classfier.positive.BUSL_1_1',
 'yalm.resources.tests.classfier.positive.Bahyph',
 'yalm.resources.tests.classfier.positive.Barr',
 'yalm.resources.tests.classfier.positive.Beerware',
 'yalm.resources.tests.classfier.positive.BitTorrent_1_0',
 'yalm.resources.tests.classfier.positive.BitTorrent_1_1',
 'yalm.resources.tests.classfier.positive.BlueOak_1_0_0',
 'yalm.resources.tests.classfier.positive.Borceux',
 'yalm.resources.tests.classfier.positive.CAL_1_0',
 'yalm.resources.tests.classfier.positive.CAL_1_0_Combined_Work_Exception',
 'yalm.resources.tests.classfier.positive.CATOSL_1_1',
 'yalm.resources.tests.classfier.positive.CC0_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_2_0',
 'yalm.resources.tests.classfier.positive.CC_BY_2_5',
 'yalm.resources.tests.classfier.positive.CC_BY_3_0',
 'yalm.resources.tests.classfier.positive.CC_BY_3_0_AT',
 'yalm.resources.tests.classfier.positive.CC_BY_3_0_US',
 'yalm.resources.tests.classfier.positive.CC_BY_4_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_2_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_2_5',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_3_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_4_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_ND_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_ND_2_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_ND_2_5',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_ND_3_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_ND_3_0_IGO',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_ND_4_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_SA_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_SA_2_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_SA_2_5',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_SA_3_0',
 'yalm.resources.tests.classfier.positive.CC_BY_NC_SA_4_0',
 'yalm.resources.tests.classfier.positive.CC_BY_ND_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_ND_2_0',
 'yalm.resources.tests.classfier.positive.CC_BY_ND_2_5',
 'yalm.resources.tests.classfier.positive.CC_BY_ND_3_0',
 'yalm.resources.tests.classfier.positive.CC_BY_ND_4_0',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_1_0',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_2_0',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_2_0_UK',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_2_1_JP',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_2_5',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_3_0',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_3_0_AT',
 'yalm.resources.tests.classfier.positive.CC_BY_SA_4_0',
 'yalm.resources.tests.classfier.positive.CC_PDDC',
 'yalm.resources.tests.classfier.positive.CDDL_1_0',
 'yalm.resources.tests.classfier.positive.CDDL_1_1',
 'yalm.resources.tests.classfier.positive.CDLA_Permissive_1_0',
 'yalm.resources.tests.classfier.positive.CDLA_Sharing_1_0',
 'yalm.resources.tests.classfier.positive.CDL_1_0',
 'yalm.resources.tests.classfier.positive.CECILL_1_0',
 'yalm.resources.tests.classfier.positive.CECILL_1_1',
 'yalm.resources.tests.classfier.positive.CECILL_2_0',
 'yalm.resources.tests.classfier.positive.CECILL_2_1',
 'yalm.resources.tests.classfier.positive.CECILL_B',
 'yalm.resources.tests.classfier.positive.CECILL_C',
 'yalm.resources.tests.classfier.positive.CERN_OHL_1_1',
 'yalm.resources.tests.classfier.positive.CERN_OHL_1_2',
 'yalm.resources.tests.classfier.positive.CERN_OHL_P_2_0',
 'yalm.resources.tests.classfier.positive.CERN_OHL_S_2_0',
 'yalm.resources.tests.classfier.positive.CERN_OHL_W_2_0',
 'yalm.resources.tests.classfier.positive.CNRI_Jython',
 'yalm.resources.tests.classfier.positive.CNRI_Python',
 'yalm.resources.tests.classfier.positive.CNRI_Python_GPL_Compatible',
 'yalm.resources.tests.classfier.positive.CPAL_1_0',
 'yalm.resources.tests.classfier.positive.CPL_1_0',
 'yalm.resources.tests.classfier.positive.CPOL_1_02',
 'yalm.resources.tests.classfier.positive.CUA_OPL_1_0',
 'yalm.resources.tests.classfier.positive.C_UDA_1_0',
 'yalm.resources.tests.classfier.positive.Caldera',
 'yalm.resources.tests.classfier.positive.ClArtistic',
 'yalm.resources.tests.classfier.positive.Condor_1_1',
 'yalm.resources.tests.classfier.positive.Crossword',
 'yalm.resources.tests.classfier.positive.CrystalStacker',
 'yalm.resources.tests.classfier.positive.Cube',
 'yalm.resources.tests.classfier.positive.DOC',
 'yalm.resources.tests.classfier.positive.DRL_1_0',
 'yalm.resources.tests.classfier.positive.DSDP',
 'yalm.resources.tests.classfier.positive.D_FSL_1_0',
 'yalm.resources.tests.classfier.positive.Dotseqn',
 'yalm.resources.tests.classfier.positive.ECL_1_0',
 'yalm.resources.tests.classfier.positive.ECL_2_0',
 'yalm.resources.tests.classfier.positive.EFL_1_0',
 'yalm.resources.tests.classfier.positive.EFL_2_0',
 'yalm.resources.tests.classfier.positive.EPICS',
 'yalm.resources.tests.classfier.positive.EPL_1_0',
 'yalm.resources.tests.classfier.positive.EPL_2_0',
 'yalm.resources.tests.classfier.positive.EUDatagrid',
 'yalm.resources.tests.classfier.positive.EUPL_1_0',
 'yalm.resources.tests.classfier.positive.EUPL_1_1',
 'yalm.resources.tests.classfier.positive.EUPL_1_2',
 'yalm.resources.tests.classfier.positive.Entessa',
 'yalm.resources.tests.classfier.positive.ErlPL_1_1',
 'yalm.resources.tests.classfier.positive.Eurosym',
 'yalm.resources.tests.classfier.positive.FSFAP',
 'yalm.resources.tests.classfier.positive.FSFUL',
 'yalm.resources.tests.classfier.positive.FSFULLR',
 'yalm.resources.tests.classfier.positive.FTL',
 'yalm.resources.tests.classfier.positive.Fair',
 'yalm.resources.tests.classfier.positive.Frameworx_1_0',
 'yalm.resources.tests.classfier.positive.FreeBSD_DOC',
 'yalm.resources.tests.classfier.positive.FreeImage',
 'yalm.resources.tests.classfier.positive.GD',
 'yalm.resources.tests.classfier.positive.GFDL_1_1',
 'yalm.resources.tests.classfier.positive.GFDL_1_1_invariants_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_1_invariants_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_1_no_invariants_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_1_no_invariants_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_1_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_1_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_2',
 'yalm.resources.tests.classfier.positive.GFDL_1_2_invariants_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_2_invariants_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_2_no_invariants_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_2_no_invariants_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_2_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_2_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_3',
 'yalm.resources.tests.classfier.positive.GFDL_1_3_invariants_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_3_invariants_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_3_no_invariants_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_3_no_invariants_or_later',
 'yalm.resources.tests.classfier.positive.GFDL_1_3_only',
 'yalm.resources.tests.classfier.positive.GFDL_1_3_or_later',
 'yalm.resources.tests.classfier.positive.GL2PS',
 'yalm.resources.tests.classfier.positive.GLWTPL',
 'yalm.resources.tests.classfier.positive.GPL_1_0',
 'yalm.resources.tests.classfier.positive.GPL_1_0_',
 'yalm.resources.tests.classfier.positive.GPL_1_0_only',
 'yalm.resources.tests.classfier.positive.GPL_1_0_or_later',
 'yalm.resources.tests.classfier.positive.GPL_2_0',
 'yalm.resources.tests.classfier.positive.GPL_2_0_',
 'yalm.resources.tests.classfier.positive.GPL_2_0_only',
 'yalm.resources.tests.classfier.positive.GPL_2_0_or_later',
 'yalm.resources.tests.classfier.positive.GPL_2_0_with_GCC_exception',
 'yalm.resources.tests.classfier.positive.GPL_2_0_with_autoconf_exception',
 'yalm.resources.tests.classfier.positive.GPL_2_0_with_bison_exception',
 'yalm.resources.tests.classfier.positive.GPL_2_0_with_classpath_exception',
 'yalm.resources.tests.classfier.positive.GPL_2_0_with_font_exception',
 'yalm.resources.tests.classfier.positive.GPL_3_0',
 'yalm.resources.tests.classfier.positive.GPL_3_0_',
 'yalm.resources.tests.classfier.positive.GPL_3_0_only',
 'yalm.resources.tests.classfier.positive.GPL_3_0_or_later',
 'yalm.resources.tests.classfier.positive.GPL_3_0_with_GCC_exception',
 'yalm.resources.tests.classfier.positive.GPL_3_0_with_autoconf_exception',
 'yalm.resources.tests.classfier.positive.Giftware',
 'yalm.resources.tests.classfier.positive.Glide',
 'yalm.resources.tests.classfier.positive.Glulxe',
 'yalm.resources.tests.classfier.positive.HPND',
 'yalm.resources.tests.classfier.positive.HPND_sell_variant',
 'yalm.resources.tests.classfier.positive.HTMLTIDY',
 'yalm.resources.tests.classfier.positive.HaskellReport',
 'yalm.resources.tests.classfier.positive.Hippocratic_2_1',
 'yalm.resources.tests.classfier.positive.IBM_pibs',
 'yalm.resources.tests.classfier.positive.ICU',
 'yalm.resources.tests.classfier.positive.IJG',
 'yalm.resources.tests.classfier.positive.IPA',
 'yalm.resources.tests.classfier.positive.IPL_1_0',
 'yalm.resources.tests.classfier.positive.ISC',
 'yalm.resources.tests.classfier.positive.ImageMagick',
 'yalm.resources.tests.classfier.positive.Imlib2',
 'yalm.resources.tests.classfier.positive.Info_ZIP',
 'yalm.resources.tests.classfier.positive.Intel',
 'yalm.resources.tests.classfier.positive.Intel_ACPI',
 'yalm.resources.tests.classfier.positive.Interbase_1_0',
 'yalm.resources.tests.classfier.positive.JPNIC',
 'yalm.resources.tests.classfier.positive.JSON',
 'yalm.resources.tests.classfier.positive.JasPer_2_0',
 'yalm.resources.tests.classfier.positive.LAL_1_2',
 'yalm.resources.tests.classfier.positive.LAL_1_3',
 'yalm.resources.tests.classfier.positive.LGPLLR',
 'yalm.resources.tests.classfier.positive.LGPL_2_0',
 'yalm.resources.tests.classfier.positive.LGPL_2_0_',
 'yalm.resources.tests.classfier.positive.LGPL_2_0_only',
 'yalm.resources.tests.classfier.positive.LGPL_2_0_or_later',
 'yalm.resources.tests.classfier.positive.LGPL_2_1',
 'yalm.resources.tests.classfier.positive.LGPL_2_1_',
 'yalm.resources.tests.classfier.positive.LGPL_2_1_only',
 'yalm.resources.tests.classfier.positive.LGPL_2_1_or_later',
 'yalm.resources.tests.classfier.positive.LGPL_3_0',
 'yalm.resources.tests.classfier.positive.LGPL_3_0_',
 'yalm.resources.tests.classfier.positive.LGPL_3_0_only',
 'yalm.resources.tests.classfier.positive.LGPL_3_0_or_later',
 'yalm.resources.tests.classfier.positive.LPL_1_0',
 'yalm.resources.tests.classfier.positive.LPL_1_02',
 'yalm.resources.tests.classfier.positive.LPPL_1_0',
 'yalm.resources.tests.classfier.positive.LPPL_1_1',
 'yalm.resources.tests.classfier.positive.LPPL_1_2',
 'yalm.resources.tests.classfier.positive.LPPL_1_3a',
 'yalm.resources.tests.classfier.positive.LPPL_1_3c',
 'yalm.resources.tests.classfier.positive.Latex2e',
 'yalm.resources.tests.classfier.positive.Leptonica',
 'yalm.resources.tests.classfier.positive.LiLiQ_P_1_1',
 'yalm.resources.tests.classfier.positive.LiLiQ_R_1_1',
 'yalm.resources.tests.classfier.positive.LiLiQ_Rplus_1_1',
 'yalm.resources.tests.classfier.positive.Libpng',
 'yalm.resources.tests.classfier.positive.Linux_OpenIB',
 'yalm.resources.tests.classfier.positive.MIT',
 'yalm.resources.tests.classfier.positive.MITNFA',
 'yalm.resources.tests.classfier.positive.MIT_0',
 'yalm.resources.tests.classfier.positive.MIT_CMU',
 'yalm.resources.tests.classfier.positive.MIT_Modern_Variant',
 'yalm.resources.tests.classfier.positive.MIT_advertising',
 'yalm.resources.tests.classfier.positive.MIT_enna',
 'yalm.resources.tests.classfier.positive.MIT_feh',
 'yalm.resources.tests.classfier.positive.MIT_open_group',
 'yalm.resources.tests.classfier.positive.MPL_1_0',
 'yalm.resources.tests.classfier.positive.MPL_1_1',
 'yalm.resources.tests.classfier.positive.MPL_2_0',
 'yalm.resources.tests.classfier.positive.MPL_2_0_no_copyleft_exception',
 'yalm.resources.tests.classfier.positive.MS_PL',
 'yalm.resources.tests.classfier.positive.MS_RL',
 'yalm.resources.tests.classfier.positive.MTLL',
 'yalm.resources.tests.classfier.positive.MakeIndex',
 'yalm.resources.tests.classfier.positive.MirOS',
 'yalm.resources.tests.classfier.positive.Motosoto',
 'yalm.resources.tests.classfier.positive.MulanPSL_1_0',
 'yalm.resources.tests.classfier.positive.MulanPSL_2_0',
 'yalm.resources.tests.classfier.positive.Multics',
 'yalm.resources.tests.classfier.positive.Mup',
 'yalm.resources.tests.classfier.positive.NAIST_2003',
 'yalm.resources.tests.classfier.positive.NASA_1_3',
 'yalm.resources.tests.classfier.positive.NBPL_1_0',
 'yalm.resources.tests.classfier.positive.NCGL_UK_2_0',
 'yalm.resources.tests.classfier.positive.NCSA',
 'yalm.resources.tests.classfier.positive.NGPL',
 'yalm.resources.tests.classfier.positive.NIST_PD',
 'yalm.resources.tests.classfier.positive.NIST_PD_fallback',
 'yalm.resources.tests.classfier.positive.NLOD_1_0',
 'yalm.resources.tests.classfier.positive.NLPL',
 'yalm.resources.tests.classfier.positive.NOSL',
 'yalm.resources.tests.classfier.positive.NPL_1_0',
 'yalm.resources.tests.classfier.positive.NPL_1_1',
 'yalm.resources.tests.classfier.positive.NPOSL_3_0',
 'yalm.resources.tests.classfier.positive.NRL',
 'yalm.resources.tests.classfier.positive.NTP',
 'yalm.resources.tests.classfier.positive.NTP_0',
 'yalm.resources.tests.classfier.positive.Naumen',
 'yalm.resources.tests.classfier.positive.NetCDF',
 'yalm.resources.tests.classfier.positive.Net_SNMP',
 'yalm.resources.tests.classfier.positive.Newsletr',
 'yalm.resources.tests.classfier.positive.Nokia',
 'yalm.resources.tests.classfier.positive.Noweb',
 'yalm.resources.tests.classfier.positive.Nunit',
 'yalm.resources.tests.classfier.positive.OCCT_PL',
 'yalm.resources.tests.classfier.positive.OCLC_2_0',
 'yalm.resources.tests.classfier.positive.ODC_By_1_0',
 'yalm.resources.tests.classfier.positive.ODbL_1_0',
 'yalm.resources.tests.classfier.positive.OFL_1_0',
 'yalm.resources.tests.classfier.positive.OFL_1_0_RFN',
 'yalm.resources.tests.classfier.positive.OFL_1_0_no_RFN',
 'yalm.resources.tests.classfier.positive.OFL_1_1',
 'yalm.resources.tests.classfier.positive.OFL_1_1_RFN',
 'yalm.resources.tests.classfier.positive.OFL_1_1_no_RFN',
 'yalm.resources.tests.classfier.positive.OGC_1_0',
 'yalm.resources.tests.classfier.positive.OGDL_Taiwan_1_0',
 'yalm.resources.tests.classfier.positive.OGL_Canada_2_0',
 'yalm.resources.tests.classfier.positive.OGL_UK_1_0',
 'yalm.resources.tests.classfier.positive.OGL_UK_2_0',
 'yalm.resources.tests.classfier.positive.OGL_UK_3_0',
 'yalm.resources.tests.classfier.positive.OGTSL',
 'yalm.resources.tests.classfier.positive.OLDAP_1_1',
 'yalm.resources.tests.classfier.positive.OLDAP_1_2',
 'yalm.resources.tests.classfier.positive.OLDAP_1_3',
 'yalm.resources.tests.classfier.positive.OLDAP_1_4',
 'yalm.resources.tests.classfier.positive.OLDAP_2_0',
 'yalm.resources.tests.classfier.positive.OLDAP_2_0_1',
 'yalm.resources.tests.classfier.positive.OLDAP_2_1',
 'yalm.resources.tests.classfier.positive.OLDAP_2_2',
 'yalm.resources.tests.classfier.positive.OLDAP_2_2_1',
 'yalm.resources.tests.classfier.positive.OLDAP_2_2_2',
 'yalm.resources.tests.classfier.positive.OLDAP_2_3',
 'yalm.resources.tests.classfier.positive.OLDAP_2_4',
 'yalm.resources.tests.classfier.positive.OLDAP_2_5',
 'yalm.resources.tests.classfier.positive.OLDAP_2_6',
 'yalm.resources.tests.classfier.positive.OLDAP_2_7',
 'yalm.resources.tests.classfier.positive.OLDAP_2_8',
 'yalm.resources.tests.classfier.positive.OML',
 'yalm.resources.tests.classfier.positive.OPL_1_0',
 'yalm.resources.tests.classfier.positive.OSET_PL_2_1',
 'yalm.resources.tests.classfier.positive.OSL_1_0',
 'yalm.resources.tests.classfier.positive.OSL_1_1',
 'yalm.resources.tests.classfier.positive.OSL_2_0',
 'yalm.resources.tests.classfier.positive.OSL_2_1',
 'yalm.resources.tests.classfier.positive.OSL_3_0',
 'yalm.resources.tests.classfier.positive.O_UDA_1_0',
 'yalm.resources.tests.classfier.positive.OpenSSL',
 'yalm.resources.tests.classfier.positive.PDDL_1_0',
 'yalm.resources.tests.classfier.positive.PHP_3_0',
 'yalm.resources.tests.classfier.positive.PHP_3_01',
 'yalm.resources.tests.classfier.positive.PSF_2_0',
 'yalm.resources.tests.classfier.positive.Parity_6_0_0',
 'yalm.resources.tests.classfier.positive.Parity_7_0_0',
 'yalm.resources.tests.classfier.positive.Plexus',
 'yalm.resources.tests.classfier.positive.PolyForm_Noncommercial_1_0_0',
 'yalm.resources.tests.classfier.positive.PolyForm_Small_Business_1_0_0',
 'yalm.resources.tests.classfier.positive.PostgreSQL',
 'yalm.resources.tests.classfier.positive.Python_2_0',
 'yalm.resources.tests.classfier.positive.QPL_1_0',
 'yalm.resources.tests.classfier.positive.Qhull',
 'yalm.resources.tests.classfier.positive.RHeCos_1_1',
 'yalm.resources.tests.classfier.positive.RPL_1_1',
 'yalm.resources.tests.classfier.positive.RPL_1_5',
 'yalm.resources.tests.classfier.positive.RPSL_1_0',
 'yalm.resources.tests.classfier.positive.RSA_MD',
 'yalm.resources.tests.classfier.positive.RSCPL',
 'yalm.resources.tests.classfier.positive.Rdisc',
 'yalm.resources.tests.classfier.positive.Ruby',
 'yalm.resources.tests.classfier.positive.SAX_PD',
 'yalm.resources.tests.classfier.positive.SCEA',
 'yalm.resources.tests.classfier.positive.SGI_B_1_0',
 'yalm.resources.tests.classfier.positive.SGI_B_1_1',
 'yalm.resources.tests.classfier.positive.SGI_B_2_0',
 'yalm.resources.tests.classfier.positive.SHL_0_5',
 'yalm.resources.tests.classfier.positive.SHL_0_51',
 'yalm.resources.tests.classfier.positive.SISSL',
 'yalm.resources.tests.classfier.positive.SISSL_1_2',
 'yalm.resources.tests.classfier.positive.SMLNJ',
 'yalm.resources.tests.classfier.positive.SMPPL',
 'yalm.resources.tests.classfier.positive.SNIA',
 'yalm.resources.tests.classfier.positive.SPL_1_0',
 'yalm.resources.tests.classfier.positive.SSH_OpenSSH',
 'yalm.resources.tests.classfier.positive.SSH_short',
 'yalm.resources.tests.classfier.positive.SSPL_1_0',
 'yalm.resources.tests.classfier.positive.SWL',
 'yalm.resources.tests.classfier.positive.Saxpath',
 'yalm.resources.tests.classfier.positive.Sendmail',
 'yalm.resources.tests.classfier.positive.Sendmail_8_23',
 'yalm.resources.tests.classfier.positive.SimPL_2_0',
 'yalm.resources.tests.classfier.positive.Sleepycat',
 'yalm.resources.tests.classfier.positive.Spencer_86',
 'yalm.resources.tests.classfier.positive.Spencer_94',
 'yalm.resources.tests.classfier.positive.Spencer_99',
 'yalm.resources.tests.classfier.positive.StandardML_NJ',
 'yalm.resources.tests.classfier.positive.SugarCRM_1_1_3',
 'yalm.resources.tests.classfier.positive.TAPR_OHL_1_0',
 'yalm.resources.tests.classfier.positive.TCL',
 'yalm.resources.tests.classfier.positive.TCP_wrappers',
 'yalm.resources.tests.classfier.positive.TMate',
 'yalm.resources.tests.classfier.positive.TORQUE_1_1',
 'yalm.resources.tests.classfier.positive.TOSL',
 'yalm.resources.tests.classfier.positive.TU_Berlin_1_0',
 'yalm.resources.tests.classfier.positive.TU_Berlin_2_0',
 'yalm.resources.tests.classfier.positive.UCL_1_0',
 'yalm.resources.tests.classfier.positive.UPL_1_0',
 'yalm.resources.tests.classfier.positive.Unicode_DFS_2015',
 'yalm.resources.tests.classfier.positive.Unicode_DFS_2016',
 'yalm.resources.tests.classfier.positive.Unicode_TOU',
 'yalm.resources.tests.classfier.positive.Unlicense',
 'yalm.resources.tests.classfier.positive.VOSTROM',
 'yalm.resources.tests.classfier.positive.VSL_1_0',
 'yalm.resources.tests.classfier.positive.Vim',
 'yalm.resources.tests.classfier.positive.W3C',
 'yalm.resources.tests.classfier.positive.W3C_19980720',
 'yalm.resources.tests.classfier.positive.W3C_20150513',
 'yalm.resources.tests.classfier.positive.WTFPL',
 'yalm.resources.tests.classfier.positive.Watcom_1_0',
 'yalm.resources.tests.classfier.positive.Wsuipa',
 'yalm.resources.tests.classfier.positive.X11',
 'yalm.resources.tests.classfier.positive.XFree86_1_1',
 'yalm.resources.tests.classfier.positive.XSkat',
 'yalm.resources.tests.classfier.positive.Xerox',
 'yalm.resources.tests.classfier.positive.Xnet',
 'yalm.resources.tests.classfier.positive.YPL_1_0',
 'yalm.resources.tests.classfier.positive.YPL_1_1',
 'yalm.resources.tests.classfier.positive.ZPL_1_1',
 'yalm.resources.tests.classfier.positive.ZPL_2_0',
 'yalm.resources.tests.classfier.positive.ZPL_2_1',
 'yalm.resources.tests.classfier.positive.Zed',
 'yalm.resources.tests.classfier.positive.Zend_2_0',
 'yalm.resources.tests.classfier.positive.Zimbra_1_3',
 'yalm.resources.tests.classfier.positive.Zimbra_1_4',
 'yalm.resources.tests.classfier.positive.Zlib',
 'yalm.resources.tests.classfier.positive.blessing',
 'yalm.resources.tests.classfier.positive.bzip2_1_0_5',
 'yalm.resources.tests.classfier.positive.bzip2_1_0_6',
 'yalm.resources.tests.classfier.positive.copyleft_next_0_3_0',
 'yalm.resources.tests.classfier.positive.copyleft_next_0_3_1',
 'yalm.resources.tests.classfier.positive.curl',
 'yalm.resources.tests.classfier.positive.diffmark',
 'yalm.resources.tests.classfier.positive.dvipdfm',
 'yalm.resources.tests.classfier.positive.eCos_2_0',
 'yalm.resources.tests.classfier.positive.eGenix',
 'yalm.resources.tests.classfier.positive.etalab_2_0',
 'yalm.resources.tests.classfier.positive.gSOAP_1_3b',
 'yalm.resources.tests.classfier.positive.gnuplot',
 'yalm.resources.tests.classfier.positive.iMatix',
 'yalm.resources.tests.classfier.positive.libpng_2_0',
 'yalm.resources.tests.classfier.positive.libselinux_1_0',
 'yalm.resources.tests.classfier.positive.libtiff',
 'yalm.resources.tests.classfier.positive.mpich2',
 'yalm.resources.tests.classfier.positive.psfrag',
 'yalm.resources.tests.classfier.positive.psutils',
 'yalm.resources.tests.classfier.positive.wxWindows',
 'yalm.resources.tests.classfier.positive.xinetd',
 'yalm.resources.tests.classfier.positive.xpp',
 'yalm.resources.tests.classfier.positive.zlib_acknowledgement',
 'yalm.resources.words',
 'yalm.template']

package_data = \
{'': ['*']}

install_requires = \
['terregex>=0.0.5,<0.0.6']

setup_kwargs = {
    'name': 'yalm',
    'version': '0.0.1a4',
    'description': 'SPDX YALM is a license matching library based on the SPDX License Matching Guideline.',
    'long_description': "# SPDX YALM: Yet Another License Matcher\nSPDX YALM is a license matching library based on [the SPDX License Matching Guideline](https://spdx.dev/license-list/matching-guidelines/).\n\nSPDX is an open standard for communicating software bill of material information. SPDX reduces redundant work by providing common formats for organizations and communities to share important data, thereby streamlining and improving compliance, security, and dependability.\n\nProject YALM is implemented as the matching guideline compliant library - this library provides a way to compare license documents and templates in a standardized way.\n\n## Features\n*    An Interface which compares text against a license template using the license matching guidelines.\n*    An Interface which returns all matching SPDX listed license ID's for any license text.\n*    An interface which compares 2 license texts and returns a boolean indicating if the 2 licenses match per the license matching guidelines.\n*    When there is no match, a return value is provided to describe where and why the license does not match.\n\n## Installation\nEnsure you have installed Python 3.9 or higher.\n\nYou can install this library via [PyPI](https://pypi.org/project/yalm/).\n```\npip install yalm\n```\n\n## License\nCopyright (c) 2021, SPDX YALM Developers.\nFiles in this repository are licensed under\n[Apache License Version 2.0](https://github.com/spdx/tools-python/blob/master/LICENSE).\n\n## Credits\nThis Project is\n- initially developed by [@anshuldutt21](https://github.com/anshuldutt21/)\nas a part of CommunityBridge Linux Foundation 2020.\n- updated by [@m1kit](https://github.com/m1kit) and released\nas a part of Google Summer of Code 2021.\n- with thanks to mentors [@goneall](https://github.com/goneall)\n",
    'author': 'mikit',
    'author_email': 'me@mikit.dev',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/m1kit/yalm-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from . import template
import xml.dom.minidom as xml


class _XmlTransformer:
  """
  Transforms a template in a license XML into a json format.
  Based on spdx/LicenseListPublisher's code.
  See: https://github.com/spdx/LicenseListPublisher/blob/master/src/org/spdx/licensexml/LicenseXmlHelper.java
  """
  HSPACE_NODE = template.TextNode(' ')
  VSPACE_NODE = template.TextNode('\n')

  def __init__(self):
    self.transformers = {
        'list': self.transform_list_node,
        'alt': self.transform_alt_node,
        'optional': self.transform_optional_node,
        'br': self.transform_break_node,
        'p': self.transform_paragraph_node,
        'titleText': self.transform_optional_node,
        'copyrightText': self.transform_copyright_node,
        'bullet': self.transform_bullet_node,
        'item': self.transform_unprocessed,
        'text': self.transform_unprocessed,
        'standardLicenseHeader': self.transform_unprocessed,
    }

  def transform_node(self, node: xml.Element) -> template.Node:
    if node.nodeType == xml.Node.TEXT_NODE:
      return self.transform_text_node(node)
    elif node.nodeType == xml.Node.ELEMENT_NODE:
      if node.tagName not in self.transformers:
        raise NotImplementedError("Unsupported node type")
      spacing = node.getAttribute('spacing') if node.hasAttribute('spacing') else 'default'
      result = []
      if spacing in ('default', 'before', 'both'):
        result.append(self.HSPACE_NODE)
      result.append(self.transformers[node.tagName](node))
      if spacing in ('after', 'both'):
        result.append(self.HSPACE_NODE)
      return template.SequentialNode(*result)
    else:
      raise NotImplementedError("Unsupported node type")

  def transform_text_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.TEXT_NODE
    return template.TextNode(node.nodeValue)

  def transform_list_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    results = []
    for child in node.childNodes:
      if child.nodeType == xml.Node.TEXT_NODE:
        results.append(self.transform_text_node(child))
      elif child.nodeType != xml.Node.ELEMENT_NODE:
        raise NotImplementedError(f"Unsupported node type")
      elif child.tagName == 'item':
        results.append(self.VSPACE_NODE)
        results.append(self.transform_unprocessed(child))
      elif child.tagName == 'list':
        results.append(self.transform_list_node(child))
      else:
        raise NotImplementedError("Unsupported node type")
    return template.SequentialNode(*results).trim()

  def transform_alt_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    original = self.transform_unprocessed(node)
    name = node.getAttribute('name')
    match = node.getAttribute('match')
    return template.VarNode(name, original, match)

  def transform_optional_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    contents = self.transform_unprocessed(node)
    return template.OptionalNode(contents)

  def transform_break_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    assert not node.childNodes
    return self.VSPACE_NODE

  def transform_paragraph_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    return template.SequentialNode(self.VSPACE_NODE, self.transform_unprocessed(node), self.VSPACE_NODE)

  def transform_copyright_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    original = self.transform_unprocessed(node)
    name = 'copyright'
    match = '.{0,5000}'
    return template.VarNode(name, original, match)

  def transform_bullet_node(self, node: xml.Element) -> template.Node:
    assert node.nodeType == xml.Node.ELEMENT_NODE
    original = self.transform_unprocessed(node)
    name = 'bullet'
    match = '.{0,20}'
    return template.VarNode(name, original, match)

  def transform_unprocessed(self, node: xml.Element) -> template.Node:
    if node.nodeType == xml.Node.TEXT_NODE:
      return self.transform_text_node(node)
    elif node.nodeType == xml.Node.ELEMENT_NODE:
      return template.SequentialNode(*[self.transform_node(child) for child in node.childNodes]).trim()
    else:
      raise NotImplementedError("Unsupported node type")


def parse_xml(text: xml.Element) -> template.Node:
  """
  Parses a dom element in the spdx-license-XML format and returns a Node.
  """
  transformer = _XmlTransformer()
  text.setAttribute('spacing', 'none')
  tree = transformer.transform_node(text)
  tree = tree.simplify()
  return tree



from importlib import resources
import json
from yalm import resources as data
from yalm.resources import words, template, regex
from yalm.template import template
import re

class KeyedDocument:
  def __init__(self, key: str, document: str):
    self.key = key
    self.document = document

  def __repr__(self) -> str:
    return repr(self.document)

  def __str__(self) -> str:
    return str(self.document)


class ResourceLoader:
  def __init__(self):
    self._meta = None
    self._equivalent_words = None
    self._expected_duplicates = None
    self._index = None
    self._words = {}
    self._template = {}
    self._regex = {}
    self._positive_samples = {}

  def _escape_license_id(self, license: str) -> str:
    return re.sub(r'\W', '_', license)

  @property
  def meta(self):
    if self._meta is None:
      self._meta = json.loads(resources.read_text(data, 'meta.json'))
    return self._meta

  @property
  def equivalent_words(self):
    if self._equivalent_words is None:
      self._equivalent_words = json.loads(resources.read_text(data, 'equivalentwords.json'))
    return self._equivalent_words

  @property
  def expected_duplicates(self):
    if self._expected_duplicates is None:
      self._expected_duplicates = json.loads(resources.read_text(data, 'expected-duplicates.json'))
    return self._expected_duplicates

  @property
  def index(self):
    if self._index is None:
      self._index = json.loads(resources.read_text(data, 'licenses.json'))
    return self._index

  def get_words(self, license) -> list[str]:
    license = self._escape_license_id(license)
    if license not in self._words:
      self._words[license] = json.loads(resources.read_text(words, f"{license}.json"))
    return self._words[license]

  def get_template(self, license) -> template.Node:
    license = self._escape_license_id(license)
    if license not in self._template:
      data = json.loads(resources.read_text(template, f"{license}.json"))
      self._template[license] = template.parse_json(data)
    return self._template[license]

  def get_regex(self, license) -> re.Pattern:
    license = self._escape_license_id(license)
    if license not in self._regex:
      self._regex[license] = re.compile(resources.read_text(regex, license), re.IGNORECASE)
    return self._regex[license]

  def get_positive_sample(self, license, key) -> KeyedDocument:
    # ensure the sample is loaded
    self.get_positive_samples(license)
    license = self._escape_license_id(license)
    return self._positive_samples[license][key]

  def get_positive_samples(self, license) -> list[KeyedDocument]:
    license = self._escape_license_id(license)
    if license not in self._positive_samples:
      path = f"yalm.resources.tests.classfier.positive.{license}"
      results = {}
      for item in resources.contents(path):
        if not item.endswith('.txt'):
          continue
        value = resources.read_text(path, item)
        results[item] = KeyedDocument(item, value)
      self._positive_samples[license] = results
    return list(self._positive_samples[license].values())

  def clear(self) -> None:
    self._meta = None
    self._equivalent_words = None
    self._expected_duplicates = None
    self._index = None
    self._words = {}
    self._template = {}
    self._regex = {}
    self._positive_samples = {}



import re
from abc import ABC, abstractmethod
from typing import Any, Optional


class TemplateParseError(Exception):
  pass


class Node(ABC):
  def __init__(self, tag: str):
    self._tag = tag

  def to_dict(self) -> Any:
    return {
        'type': self._tag,
    }

  def _is_empty(self) -> bool:
    return False

  def trim(self, char: Optional[str] = None, l: bool = True, r: bool = True) -> 'Node':
    return self

  def simplify(self) -> 'Node':
    return self


class SequentialNode(Node):
  def __init__(self, *nodes: Node) -> None:
    super().__init__('sequential')
    self.nodes = list(nodes)

  def to_dict(self) -> Any:
    return [node.to_dict() for node in self.nodes]

  def _is_empty(self) -> bool:
    return all(node._is_empty() for node in self.nodes)

  def trim(self, char: Optional[str] = None, l: bool = True, r: bool = True) -> Node:
    # first, we need to simplify to merge consecutive text nodes
    t = self.simplify()
    if not isinstance(t, SequentialNode):
      return t.trim(char=char, l=l, r=r)
    if t._is_empty():
      return t
    if isinstance(t.nodes[0], TextNode):
      t.nodes[0] = t.nodes[0].trim(char=char, l=l, r=False)
    if isinstance(t.nodes[-1], TextNode):
      t.nodes[-1] = t.nodes[-1].trim(char=char, l=False, r=r)
    return t

  def simplify(self) -> Node:
    # Normalize
    self.nodes = [node.simplify() for node in self.nodes if not node._is_empty()]
    # Flatten nodes
    nodes_flat = []
    for node in self.nodes:
      if isinstance(node, self.__class__):
        # flatten nested seq
        nodes_flat += node.nodes
      else:
        nodes_flat.append(node)
    # Merge nodes
    nodes_merged = []
    for node in nodes_flat:
      if nodes_merged and isinstance(node, TextNode) and isinstance(nodes_merged[-1], TextNode):
        # concat texts
        nodes_merged[-1] = TextNode(nodes_merged[-1].text + node.text)
      else:
        nodes_merged.append(node)
    self.nodes = nodes_merged
    # If there is only one node, this node can be "unwrapped"
    if len(self.nodes) == 1:
      return self.nodes[0]
    return self

  def __str__(self):
    return ''.join(map(str, self.nodes))

  def __repr__(self):
    return f"SequentialNode({', '.join(map(repr, self.nodes))})"


class TextNode(Node):
  _regex_hspace = re.compile(r'[^\S\n\v\f\r\u2028\u2029]+')
  _regex_vspace = re.compile(r'\s*[\n\v\f\r\u2028\u2029]\s*')

  def __init__(self, text: str):
    super().__init__('text')
    self.text = text

  def to_dict(self):
    return {
        **super().to_dict(),
        'content': self.text,
    }

  def _is_empty(self):
    return self.text == ''

  def trim(self, char: Optional[str] = None, l: bool = True, r: bool = True) -> 'TextNode':
    if l:
      self.text = self.text.lstrip(char)
    if r:
      self.text = self.text.rstrip(char)
    return self

  def __str__(self):
    return re.escape(self.text)

  def __repr__(self):
    return f"TextNode({repr(self.text)})"


class OptionalNode(Node):
  def __init__(self, content: Node):
    super().__init__('optional')
    self.content = content

  def to_dict(self):
    return {
        **super().to_dict(),
        'content': self.content.to_dict(),
    }

  def _is_empty(self):
    return self.content._is_empty()

  def simplify(self):
    self.content = self.content.simplify()
    return self

  def __str__(self):
    return f"(?:{self.content})?"

  def __repr__(self):
    return f"OptionalNode({repr(self.content)})"


class VarNode(Node):
  def __init__(self, name: str, original: Node, pattern: str):
    super().__init__('var')
    self.name = name
    self.original = original
    self.pattern = pattern

  def to_dict(self):
    return {
        **super().to_dict(),
        'name': self.name,
        'content': self.original.to_dict(),
        'pattern': self.pattern,
    }

  def simplify(self):
    self.original = self.original.simplify()
    return self

  def __str__(self):
    return f"(?:{self.pattern})"

  def __repr__(self):
    return f"VarNode({repr(self.name)}, {repr(self.original)}, {repr(self.pattern)})"


def parse_json(node: Any) -> Node:
  """
  Deserialize nodes in json format.
  """
  if type(node) == list:
    contents = [parse_json(c) for c in node]
    return SequentialNode(*contents)
  elif node['type'] == 'text':
    return TextNode(node['content'])
  elif node['type'] == 'optional':
    content = parse_json(node['content'])
    return OptionalNode(content)
  elif node['type'] == 'var':
    name = node['name']
    content = parse_json(node['content'])
    pattern = node['pattern']
    return VarNode(name, content, pattern)
  else:
    raise NotImplementedError('Unsupported node type')
